import React,{ Component } from 'react'
import { autobind } from 'core-decorators'
import Content from '../../componentes/pages/content'
import Panel from '../../componentes/panel'
import {ButtonGroup, Row, Col} from 'react-bootstrap'
import Gantt from '../../componentes/gantt'
import GanttToolbar from '../../componentes/gantt/gantt-toolbar'
import MessageArea from '../../componentes/gantt/gantt-message'
import Edit from '../../componentes/edit/edit-dialog'
import { Schema } from '../../utils/form'
import axios from 'axios'
import events from 'events'
import AcaoEdit from '../acao/edit'
import Button from '../../componentes/button'
import ContentManager from '../../componentes/content-manager'
import moment from 'moment'
import MdiIcon from '../../componentes/mdi-icon'


let schema = Schema.object({
  id: Schema.number().label("id"),
})

@autobind
class FormEdit extends React.Component  {

  constructor(props) {
    super(props)

    this.state = {
      currentZoom: 'Months',
      messages: [],
    }
    this.handleZoomChange = this.handleZoomChange.bind(this)
    this.logTaskUpdate = this.logTaskUpdate.bind(this)
    this.logLinkUpdate = this.logLinkUpdate.bind(this)
  }

  addMessage(message) {
    let messages = this.state.messages.slice()
    let prevKey = messages.length ? messages[0].key: 0

    messages.unshift({key: prevKey + 1, message})
    if(messages.length > 40){
      messages.pop()
    }
    this.setState({messages})
  }

  logTaskUpdate(id, mode, task) {
    let text = task && task.text ? ` (${task.text})`: ''
    let message = `Task ${mode}: ${id} ${text}`
    this.addMessage(message)
  }

  logLinkUpdate(id, mode, link) {
    let message = `Link ${mode}: ${id}`
    if (link) {
      message += ` (source: ${link.source}, target: ${link.target})`
    }
    this.addMessage(message)
  }

  handleZoomChange(zoom) {
    this.setState({
      currentZoom: zoom
    })
  }

  componentDidMount() {
    this.atualizar()
  }

  atualizar(){
    this.setState({
      'isLoading': true
    })
    if (!this.props.id) {
      axios.post(`/Acao/ObterAcoesPorItem`, {
        itemId : this.props.itemId,
        parameters: this.props.parameters

      }).then((response) => {
        this.setState({
          loaded: true,
          isLoading: false,
          acoes:response.data
        })
      })
    }
  }

  handleProgress(progress){return Math.round(progress)}

  handleDuration(dataInicioPrevista,dataFimPrevista){
    let inicio = moment(dataInicioPrevista)
    let fim = moment(dataFimPrevista)
    let duracao = fim.diff(inicio, 'days')
    if (duracao<1){duracao=1}
    return duracao
  }

  handlePercentualPrevisto(durationTotal,dataInicioPrevista){
    let now = new Date()
    let hoje = moment(now)
    let inicio = moment(dataInicioPrevista)
    let diasCorridos = hoje.diff(inicio,'days')
    return Math.round((diasCorridos/durationTotal)*100)
  }

  handleValorNulo(valor){
    if (valor == null){
      return ''
    }else{
      return valor
    }
  }

  handleCloseAcao(){
    this.atualizar()
  }

  handleNovaAcao(){
    ContentManager.addContent(
      <AcaoEdit
        parameters={{itemId:this.props.itemId}}
        handleClose={this.handleCloseAcao}>
      </AcaoEdit>)
  }

  handleTaskUpdate(item,teste){
    console.log("not implemented yet")
  }
  render (){
    let acoes = this.state.acoes && this.state.acoes.map((acao) => (
      {
        id: acao.id,
        text: acao.descricao,
        codigo:acao.codigo,
        start_date: acao.dataInicioPrevista,
        farol_acao:acao.farolAcao,
        duration: this.handleDuration(acao.dataInicioPrevista,acao.dataFimPrevista),
        progress: this.handleProgress(acao.percentualRealizado),
        percentual_previsto:this.handlePercentualPrevisto(
          this.handleDuration(acao.dataInicioPrevista,acao.dataFimPrevista),
          acao.dataInicioPrevista
        ),
        holder:acao.responsavel.nome,
        status_acao:acao.status.descricao,
        baseline_inicial:this.handleValorNulo(acao.baselineInicial),
        baseline_final:this.handleValorNulo(acao.baselineFinal),
        data_inicio_realizada:this.handleValorNulo(acao.dataInicioRealizada),
        data_fim_realizada:this.handleValorNulo(acao.dataFimRealizada),
        observacao:this.handleValorNulo(acao.observacao),
        como:this.handleValorNulo(acao.como),
        onde:this.handleValorNulo(acao.onde),
        porque:this.handleValorNulo(acao.porque)

      }
    ))

    let data = {
      data:[].concat(acoes)
    }

    return(
      <div>
      <Panel style={{marginTop: 15, height: 700}}>
        <Row>
          <Col md={12}>
            <Content  {...this.props}>

              <Row>
                <Col md={9}>
                  <GanttToolbar
                    zoom={this.state.currentZoom}
                    onZoomChange={this.handleZoomChange}/>
                </Col>
                <Col md={3}>
                  <Button
                    type="primary"
                    className="pull-right"
                    onClick={this.handleNovaAcao}>
                    Nova ação
                  </Button>
                </Col>
              </Row>

              <Row>
                <div className="gantt-container">
                  {
                   acoes &&
                    <Gantt
                      tasks={data}
                      zoom={this.state.currentZoom}
                      onTaskUpdated={this.handleTaskUpdate}
                      onLinkUpdated={this.logLinkUpdate}
                    />
                  }
                </div>
              </Row>
              <MessageArea
                messages={this.state.messages}
              />
            </Content>
          </Col>

        </Row>
      </Panel>
    </div>
    )
  }
}

export default class GanttEdit extends React.Component{
  render(){
    return(
      <Edit
        url="/Gantt"
        titulo="Gráfico de Gantt"
        formComponent={FormEdit}
        schema={schema}
        width={1500}
        height={700}
        {...this.props} />
    )
  }
}
